<?php
 /**
# mod_jvslidepro - JV Slide Pro
# @versions: 1.5.x,1.6.x,1.7.x,2.5.x
# ------------------------------------------------------------------------
# author    Open Source Code Solutions Co
# copyright Copyright (C) 2011 joomlavi.com. All Rights Reserved.
# @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL or later.
# Websites: http://www.joomlavi.com
# Technical Support:  http://www.joomlavi.com/my-tickets.html
-------------------------------------------------------------------------*/
defined('_JEXEC') or die('Restricted access');
if(!class_exists('JVImageEditor')){
    class JVImageEditor{
        private 
            $imagePath = '',
            $options,
            $imageSource,
            $details,
            $cachepath
        ;
        
        
        function __construct($image,$options = array()){
            $this->cachepath = JPATH_SITE.DS.'modules'.DS.'mod_jvslide3'.DS.'assets'.DS.'cache'.DS;
            $this->imagePath = $image;
            $this->options = $options;
            $this->options['RGB'] = $this->HexToRGB($options['background']);
            if($this->getCache()) die;
            $this->imageSource = $this->createSource();
            $this->resize($options['resize']);
            $this->crop($options['crop']);
            $this->corners($options['corners']);
            if($options['reflect']['@selected'] == 'yes') $this->reflect($options['reflect']['yes']);
            $this->out();
            $this->setCache();
            imagedestroy($this->imageSource);
        }
        
        function createSource(){
            $this->details = getimagesize($this->imagePath);
            switch ($this->details[2])
            {
                case 1: return imagecreatefromgif($this->imagePath); break;
                case 2: return imagecreatefromjpeg($this->imagePath); break;
                case 3: return imagecreatefrompng($this->imagePath); break; 
                default: echo 'Unsupported image file format.'; exit();
            }
        }
        
        function resize($options){
            if(!$options['width'] && !$options['height']) return;
            
            if(!isset($options['width'])) $options['width'] = $this->details[0];
            if(!isset($options['height'])) $options['height'] = $this->details[1];
            
            if($options['width'] == 0){
                $options['width'] = $options['height']/$this->details[1] * $this->details[0];
            }elseif($options['height'] == 0){
                $options['height'] = $options['width']/$this->details[0] * $this->details[1];
            }
            $options['width'] = (int) $options['width'];
            $options['height'] = (int) $options['height'];
            
            $output = imagecreatetruecolor($options['width'],$options['height']);
            ImageCopyResized($output,$this->imageSource,0,0,0,0,$options['width'],$options['height'],$this->details[0],$this->details[1]);
            $this->imageSource = $output;
            $this->details[0] = $options['width'];
            $this->details[1] = $options['height'];
        }
        
        function crop($options){
            if(!$options['width'] && !$options['height']) return;
            if(!$options['width'] || $options['width'] > $this->details[0]) $options['width'] = $this->details[0];
            if(!$options['height'] || $options['height'] > $this->details[1]) $options['height'] = $this->details[1];
            $top = (int) ($this->details[1]/2 - $options['height']/2);
            $left = (int) ($this->details[0] / 2 - $options['width']/2);
            
            $output = imagecreatetruecolor($options['width'],$options['height']);
            
            imagecopyresampled($output, $this->imageSource, 0, 0,$left, $top, $options['width'],$options['height'], $options['width'],$options['height']);
            $this->imageSource = $output;
            $this->details[0] = $options['width'];
            $this->details[1] = $options['height'];
        }
        
        function corners($options){
            $radiuss = $this->options['corners'];
            foreach($radiuss as $k => $v) $radiuss[$k] = (int) $v;
            if(!$radiuss['lt'] && !$radiuss['tr'] && !$radiuss['rb'] && !$radiuss['bl'] ) return;
            $dest = $this->imageSource;
            list($width,$height) = $this->details;
            $q = floor(400000/($width*$height));
            $q = $q < 1? 1: $q;
            $RGB = array(
                'r' => 127,
                'g' => 127,
                'b' => 127
            );
            foreach($radiuss as $k => $v) $radiuss[$k] = $v * $q;
            $new_width=$width*$q;
            $new_height=$height*$q;
            
            $magnified=imagecreatetruecolor($new_width, $new_height);
            //imagesavealpha($magnified, true);
            imagealphablending($magnified, false);
            imagecopyresampled($magnified, $dest, 0,0, 0,0, $new_width,$new_height, $width,$height);

            $colorcode = imagecolorallocatealpha($magnified, $RGB['r'], $RGB['g'], $RGB['b'], 127);
            
            $radius = (int)$radiuss['lt'];
            if($radius > 1){
                imagearc($magnified, $radius-1, $radius-1, $radius*2, $radius*2, 180, 270, $colorcode);
                imagefilltoborder($magnified, 0, 0, $colorcode, $colorcode);
            }
            
            $radius = (int)$radiuss['tr'];
            if($radius > 1){
                imagearc($magnified, $new_width-$radius, $radius-1, $radius*2, $radius*2, 270, 0, $colorcode);
                imagefilltoborder($magnified, $new_width-1, 0, $colorcode, $colorcode);
            }
            $radius = (int)$radiuss['rb'];
            if($radius > 1){
                imagearc($magnified, $new_width-$radius, $new_height-$radius, $radius*2, $radius*2, 0, 90, $colorcode);
                imagefilltoborder($magnified, $new_width-1, $new_height-1, $colorcode, $colorcode);
            }
            $radius = (int)$radiuss['bl'];
            if($radius > 1){
                imagearc($magnified, $radius-1, $new_height-$radius, $radius*2, $radius*2, 90, 180, $colorcode);
                imagefilltoborder($magnified, 0, $new_height-1, $colorcode, $colorcode);
            }
        
            $dest=imagecreatetruecolor($width, $height);
            imagefill($dest,0,0,$colorcode);
            imagecopyresampled($dest, $magnified, 0,0, 0,0, $width,$height, $new_width,$new_height);
            //imagesavealpha($dest, true);
            $this->imageSource=$dest;
                
        }
        
        function reflect($options){
           
            $source = $this->imageSource;
            $image_details = $this->details;
            $RGB = array(
                'r' => 127,
                'g' => 127,
                'b' => 127
            );
            list($width,$height) = $image_details;
            
            $mime = $image_details['mime'];
            $alpha_start = $options['alpha']['start'] ;
            $alpha_end = $options['alpha']['end']; 
            
            $newsize = $options['size'] * $height;
            if ($options['size'] > 1) $newsize = $options['size'];
            $elasticity = (int) ($options['elasticity'] * $newsize);
            
            $output = imagecreatetruecolor($width, $newsize);
            $buffer = imagecreatetruecolor($width, $newsize);
            //imagesavealpha($output, true);
            imagealphablending($output, false);
            //imagesavealpha($buffer, true);
            imagealphablending($buffer, false);
            imagecopy($output, $source, 0, 0, 0, $height - $newsize, $width, $newsize);
            for ($y = 0; $y < $newsize; $y++) imagecopy($buffer, $output, 0, $y, 0, $newsize - $y - 1, $width, 1);
            $output = $buffer;
            $alpha_length = abs($alpha_start - $alpha_end);
            imagelayereffect($output, IMG_EFFECT_OVERLAY);
            for ($y = 0; $y <= $newsize; $y++) {
                $pct = $y / $newsize;
                if ($alpha_start > $alpha_end) $alpha = (int) ($alpha_start - ($pct * $alpha_length));
                else $alpha = (int) ($alpha_start + ($pct * $alpha_length));
                $alpha = 127 - $alpha;
                imagefilledrectangle($output, 0, $y, $width, $y, imagecolorallocatealpha($output, $RGB['r'], $RGB['g'], $RGB['b'], $alpha));
            }
            $this->details[1] = $newHeight = $height+$elasticity + $options['space'];
            $finaloutput = imagecreatetruecolor($width, $newHeight);
            //imagesavealpha($finaloutput, true);
            imagealphablending($finaloutput, false);
            $trans_colour = imagecolorallocatealpha($finaloutput, $RGB['r'], $RGB['g'], $RGB['b'], 127);
            imagefill($finaloutput, 0, 0, $trans_colour);
            
            imagecopy($finaloutput, $source, 0, 0, 0, 0, $width, $height);
            imagecopyresized($finaloutput, $output, 0, $height + $options['space'], 0, 0, $width, $elasticity,$width,$newsize);
            
            $this->imageSource = $finaloutput;
            return;
        }
        
        function setBackground(){
            list($width,$height) = $this->details;
            $RGB = $this->options['RGB'];
            $dest=imagecreatetruecolor($width, $height);
            $solid_colour = imagecolorallocate( $dest,$RGB['r'],$RGB['g'],$RGB['b']);
            imagefill($dest,0,0,$solid_colour);
            imagecopyresampled($dest, $this->imageSource, 0,0, 0,0, $width,$height, $width,$height);
            $this->imageSource=$dest;
        }
        function out(){
            switch($this->options['outputas']){
                case 'jpg': 
                    header("Content-type: image/jpeg");
                    $this->setBackground();
                    imagejpeg($this->imageSource);
                    break;
                case 'png':  
                    header("Content-type: image/png");
                    imagesavealpha($this->imageSource, true);
                    imagepng($this->imageSource);
                    break;
            }
        }
        
        function setCache(){
            switch($this->options['outputas']){
                case 'jpg': 
                    @imagejpeg($this->imageSource,$this->getCacheName());
                    break;
                case 'png':
                    @imagepng($this->imageSource,$this->getCacheName());
                    break;
            }
        }
        function getCache(){
            $cachename = $this->getCacheName();
            if(!is_file($cachename)) return false;
            if($this->options['outputas'] == 'jpg') $this->options['outputas'] = 'jpeg';
            header("Content-type: image/{$this->options['outputas']}");
            readfile($cachename);
            return true;
        }
        
        private function getCacheName(){
            return JPATH_SITE.DS.'modules'.DS.'mod_jvslidepro'.DS.'assets'.DS.'cache'.DS.md5($this->imagePath.json_encode($this->options)).'.'.$this->options['outputas'];
        }
        
        private function extend(&$a,$b){
            foreach($b as $k => $v){
                $a[$k] = $v;
            }
            return $a;
        }
        
        function HexToRGB($hex) {
            $hex = ereg_replace("#", "", $hex);
            switch(strlen($hex)){
                case 3:
                    $color = array(
                        'r' => hexdec(substr($hex, 0, 1)),
                        'g' => hexdec(substr($hex, 1, 1)),
                        'b' => hexdec(substr($hex, 2, 1))
                    );
                    break;
                case 6: 
                    $color = array(
                        'r' => hexdec(substr($hex, 0, 2)),
                        'g' => hexdec(substr($hex, 2, 2)),
                        'b' => hexdec(substr($hex, 4, 2))
                    );
                    break;
                default:
                     $color = array(
                        'r' => 0,
                        'g' => 0,
                        'b' => 0
                    );
            }
            return $color;
        }
    }
}
?>